@@ -1,6 +1,6 @@
 import json
 import os
-from hashlib import blake2b
+from hashlib import blake2b, sha256
 from http.cookies import SimpleCookie
 
 import jwt
@@ -48,11 +48,26 @@ def request_verification_flow(headers):
 	
 	code, message = validate_jwt(jwt_token, secret=os.environ['JWT_KEY'])
 	if code != 200:
-		return {'statusCode': code, 'body': json.dumps({'message': message}), 'headers': {ACAO: origin}}
+		return {'statusCode': code, 'body': json.dumps({'message': message}), 'headers': {ACAO: origin, ACAC: True}}
 	
 	return message, origin
 
 
+def outsiders_verification_flow(oms, event, body, marker):
+	path = event['path']
+	
+	if 'trading_view' in path and str(body['marker']) != str(marker):
+		return False
+	elif 'kite' in path:
+		h = sha256(body['order_id'].encode() + body['order_timestamp'].encode() + oms.api_secret.encode())
+		checksum = h.hexdigest()
+		
+		if checksum != body['checksum']:
+			return False
+	
+	return True
+
+
 def get_hashed_password(password, key):
 	h = blake2b(key=key.encode(), digest_size=16)
 	h.update(password.encode())
